@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2009 Edgewall Software
+# Copyright (C) 2006-2013 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -15,30 +15,34 @@ from __future__ import with_statement
 
 import doctest
 import os.path
+import pkg_resources
 import random
 import re
+import shutil
+import sys
 import tempfile
 import unittest
 
+import trac
+import trac.tests.compat
 from trac import util
-from trac.util.tests import concurrency, datefmt, presentation, text, html
+from trac.util.tests import concurrency, datefmt, presentation, text, \
+                            translation, html
 
 
 class AtomicFileTestCase(unittest.TestCase):
 
     def setUp(self):
-        self.path = os.path.join(tempfile.gettempdir(), 'trac-tempfile')
+        self.dir = tempfile.mkdtemp()
+        self.path = os.path.join(self.dir, 'trac-tempfile')
 
     def tearDown(self):
-        try:
-            os.unlink(self.path)
-        except OSError:
-            pass
+        shutil.rmtree(self.dir)
 
     def test_non_existing(self):
         with util.AtomicFile(self.path) as f:
             f.write('test content')
-        self.assertEqual(True, f.closed)
+        self.assertTrue(f.closed)
         self.assertEqual('test content', util.read_file(self.path))
 
     def test_existing(self):
@@ -46,9 +50,21 @@ class AtomicFileTestCase(unittest.TestCase):
         self.assertEqual('Some content', util.read_file(self.path))
         with util.AtomicFile(self.path) as f:
             f.write('Some new content')
-        self.assertEqual(True, f.closed)
+        self.assertTrue(f.closed)
         self.assertEqual('Some new content', util.read_file(self.path))
 
+    if os.name != 'nt':
+        def test_symbolic_link(self):
+            link_path = os.path.join(self.dir, 'trac-tempfile-link')
+            os.symlink(self.path, link_path)
+
+            with util.AtomicFile(link_path) as f:
+                f.write('test content')
+
+            self.assertTrue(os.path.islink(link_path))
+            self.assertEqual('test content', util.read_file(link_path))
+            self.assertEqual('test content', util.read_file(self.path))
+
     if util.can_rename_open_file:
         def test_existing_open_for_reading(self):
             util.create_file(self.path, 'Initial file content')
@@ -56,8 +72,8 @@ class AtomicFileTestCase(unittest.TestCase):
             with open(self.path) as rf:
                 with util.AtomicFile(self.path) as f:
                     f.write('Replaced content')
-            self.assertEqual(True, rf.closed)
-            self.assertEqual(True, f.closed)
+            self.assertTrue(rf.closed)
+            self.assertTrue(f.closed)
             self.assertEqual('Replaced content', util.read_file(self.path))
 
     # FIXME: It is currently not possible to make this test pass on all
@@ -70,18 +86,18 @@ class AtomicFileTestCase(unittest.TestCase):
         self.path = os.path.join(tempfile.gettempdir(), u'träc-témpfilè')
         with util.AtomicFile(self.path) as f:
             f.write('test content')
-        self.assertEqual(True, f.closed)
+        self.assertTrue(f.closed)
         self.assertEqual('test content', util.read_file(self.path))
 
 
 class PathTestCase(unittest.TestCase):
 
     def assert_below(self, path, parent):
-        self.assert_(util.is_path_below(path.replace('/', os.sep),
-                                        parent.replace('/', os.sep)))
+        self.assertTrue(util.is_path_below(path.replace('/', os.sep),
+                                           parent.replace('/', os.sep)))
 
     def assert_not_below(self, path, parent):
-        self.assert_(not util.is_path_below(path.replace('/', os.sep),
+        self.assertFalse(util.is_path_below(path.replace('/', os.sep),
                                             parent.replace('/', os.sep)))
 
     def test_is_path_below(self):
@@ -93,8 +109,8 @@ class PathTestCase(unittest.TestCase):
         self.assert_not_below('/svn/project2/sub/repos', '/svn/project1')
         self.assert_not_below('/svn/project1/../project2/repos',
                               '/svn/project1')
-        self.assert_(util.is_path_below('repos', os.path.join(os.getcwd())))
-        self.assert_(not util.is_path_below('../sub/repos',
+        self.assertTrue(util.is_path_below('repos', os.path.join(os.getcwd())))
+        self.assertFalse(util.is_path_below('../sub/repos',
                                             os.path.join(os.getcwd())))
 
 
@@ -162,25 +178,153 @@ class SafeReprTestCase(unittest.TestCase):
         self.assertRaises(Exception, repr, eh_ix())
         sr = util.safe_repr(eh_ix())
         sr = re.sub('[A-F0-9]{4,}', 'ADDRESS', sr)
-        sr = re.sub(r'__main__|trac\.util\.tests', 'MODULE', sr)
+        sr = re.sub(r'__main__|trac\.util\.tests(\.__init__)?', 'MODULE', sr)
         self.assertEqual("<MODULE.eh_ix object at 0xADDRESS "
                          "(repr() error: TypeError: unsupported operand "
                          "type(s) for +: 'int' and 'str')>", sr)
 
 
+class SetuptoolsUtilsTestCase(unittest.TestCase):
+
+    def test_get_module_path(self):
+        self.assertEqual(util.get_module_path(trac),
+                         util.get_module_path(util))
+
+    def test_get_pkginfo_trac(self):
+        pkginfo = util.get_pkginfo(trac)
+        self.assertEqual(trac.__version__, pkginfo.get('version'))
+        self.assertNotEqual({}, pkginfo)
+
+    def test_get_pkginfo_non_toplevel(self):
+        from trac import core
+        import tracopt
+        pkginfo = util.get_pkginfo(trac)
+        self.assertEqual(pkginfo, util.get_pkginfo(util))
+        self.assertEqual(pkginfo, util.get_pkginfo(core))
+        self.assertEqual(pkginfo, util.get_pkginfo(tracopt))
+
+    def test_get_pkginfo_genshi(self):
+        try:
+            import genshi
+            import genshi.core
+            dist = pkg_resources.get_distribution('Genshi')
+        except:
+            pass
+        else:
+            pkginfo = util.get_pkginfo(genshi)
+            self.assertNotEqual({}, pkginfo)
+            self.assertEqual(pkginfo, util.get_pkginfo(genshi.core))
+
+    def test_get_pkginfo_babel(self):
+        try:
+            import babel
+            import babel.core
+            dist = pkg_resources.get_distribution('Babel')
+        except:
+            pass
+        else:
+            pkginfo = util.get_pkginfo(babel)
+            self.assertNotEqual({}, pkginfo)
+            self.assertEqual(pkginfo, util.get_pkginfo(babel.core))
+
+    def test_get_pkginfo_mysqldb(self):
+        # MySQLdb's package name is "MySQL-Python"
+        try:
+            import MySQLdb
+            import MySQLdb.cursors
+            dist = pkg_resources.get_distribution('MySQL-Python')
+            dist.get_metadata('top_level.txt')
+        except:
+            pass
+        else:
+            pkginfo = util.get_pkginfo(MySQLdb)
+            self.assertNotEqual({}, pkginfo)
+            self.assertEqual(pkginfo, util.get_pkginfo(MySQLdb.cursors))
+
+    def test_get_pkginfo_psycopg2(self):
+        # python-psycopg2 deb package doesn't provide SOURCES.txt and
+        # top_level.txt
+        try:
+            import psycopg2
+            import psycopg2.extensions
+            dist = pkg_resources.get_distribution('psycopg2')
+        except:
+            pass
+        else:
+            pkginfo = util.get_pkginfo(psycopg2)
+            self.assertNotEqual({}, pkginfo)
+            self.assertEqual(pkginfo, util.get_pkginfo(psycopg2.extensions))
+
+
+class LazyClass(object):
+    @util.lazy
+    def f(self):
+        return object()
+
+
+class LazyTestCase(unittest.TestCase):
+
+    def setUp(self):
+        self.obj = LazyClass()
+
+    def test_lazy_get(self):
+        f = self.obj.f
+        self.assertTrue(self.obj.f is f)
+
+    def test_lazy_set(self):
+        self.obj.f = 2
+        self.assertEqual(2, self.obj.f)
+
+    def test_lazy_del(self):
+        f = self.obj.f
+        del self.obj.f
+        self.assertFalse(self.obj.f is f)
+
+
+class FileTestCase(unittest.TestCase):
+
+    def setUp(self):
+        self.dir = tempfile.mkdtemp()
+        self.filename = os.path.join(self.dir, 'trac-tempfile')
+        self.data = 'Lorem\ripsum\ndolor\r\nsit\namet,\rconsectetur\r\n'
+
+    def tearDown(self):
+        shutil.rmtree(self.dir)
+
+    def test_create_and_read_file(self):
+        util.create_file(self.filename, self.data, 'wb')
+        with open(self.filename, 'rb') as f:
+            self.assertEqual(self.data, f.read())
+        self.assertEqual(self.data, util.read_file(self.filename, 'rb'))
+
+    def test_touch_file(self):
+        util.create_file(self.filename, self.data, 'wb')
+        util.touch_file(self.filename)
+        with open(self.filename, 'rb') as f:
+            self.assertEqual(self.data, f.read())
+
+    def test_missing(self):
+        util.touch_file(self.filename)
+        self.assertTrue(os.path.isfile(self.filename))
+        self.assertEqual(0, os.path.getsize(self.filename))
+
 
 def suite():
     suite = unittest.TestSuite()
-    suite.addTest(unittest.makeSuite(AtomicFileTestCase, 'test'))
-    suite.addTest(unittest.makeSuite(PathTestCase, 'test'))
-    suite.addTest(unittest.makeSuite(RandomTestCase, 'test'))
-    suite.addTest(unittest.makeSuite(ContentDispositionTestCase, 'test'))
-    suite.addTest(unittest.makeSuite(SafeReprTestCase, 'test'))
+    suite.addTest(unittest.makeSuite(AtomicFileTestCase))
+    suite.addTest(unittest.makeSuite(PathTestCase))
+    suite.addTest(unittest.makeSuite(RandomTestCase))
+    suite.addTest(unittest.makeSuite(ContentDispositionTestCase))
+    suite.addTest(unittest.makeSuite(SafeReprTestCase))
+    suite.addTest(unittest.makeSuite(SetuptoolsUtilsTestCase))
+    suite.addTest(unittest.makeSuite(LazyTestCase))
+    suite.addTest(unittest.makeSuite(FileTestCase))
     suite.addTest(concurrency.suite())
     suite.addTest(datefmt.suite())
     suite.addTest(presentation.suite())
     suite.addTest(doctest.DocTestSuite(util))
     suite.addTest(text.suite())
+    suite.addTest(translation.suite())
     suite.addTest(html.suite())
     return suite
 
