@@ -1,15 +1,56 @@
 # -*- coding: utf-8 -*-
-from datetime import datetime
+#
+# Copyright (C) 2006-2013 Edgewall Software
+# All rights reserved.
+#
+# This software is licensed as described in the file COPYING, which
+# you should have received as part of this distribution. The terms
+# are also available at http://trac.edgewall.org/wiki/TracLicense.
+#
+# This software consists of voluntary contributions made by many
+# individuals. For the exact contribution history, see the revision
+# history and logs, available at http://trac.edgewall.org/log/.
+
+from StringIO import StringIO
+import os
+import shutil
+import tempfile
 import unittest
 
-from trac.config import Option
+from trac.config import Option, ListOption, IntOption, BoolOption
 from trac.test import locale_en
-from trac.util.datefmt import format_date, utc
+from trac.util.datefmt import datetime_now, format_date, utc
 from trac.wiki.model import WikiPage
 from trac.wiki.tests import formatter
 
+
+def add_pages(tc, names):
+    now = datetime_now(utc)
+    for name in names:
+        w = WikiPage(tc.env)
+        w.name = name
+        w.text = '--'
+        w.save('joe', 'the page ' + name, '::1', now)
+
+
 # == [[Image]]
 
+def image_setup(tc):
+    add_pages(tc, ['page:fr'])
+    from trac.attachment import Attachment
+    tc.env.path = tempfile.mkdtemp(prefix='trac-tempenv-')
+    attachment = Attachment(tc.env, 'wiki', 'page:fr')
+    attachment.description = "image in page:fr"
+    attachment.insert('img.png', StringIO(''), 0, 2)
+    htdocs_location = 'http://assets.example.org/common'
+    tc.context.req.chrome['htdocs_location'] = htdocs_location
+    tc.env.config.set('trac', 'htdocs_location', htdocs_location)
+
+def image_teardown(tc):
+    shutil.rmtree(os.path.join(tc.env.path, 'files'))
+    os.rmdir(tc.env.path) # there was only 'files' below tc.env.path
+    tc.env.reset_db()
+
 # Note: using `« test »` string in the following tests for checking
 #       unicode robustness and whitespace support (first space is
 #       normal ASCII SPACE, second is Unicode NO-BREAK SPACE).
@@ -22,6 +63,14 @@ IMAGE_MACRO_TEST_CASES = u"""
 <a style="padding:0; border:none" href="/browser/%C2%AB%20test%C2%A0%C2%BB.png"><img src="/browser/%C2%AB%20test%C2%A0%C2%BB.png?format=raw" alt="source:« test ».png" title="source:« test ».png" /></a>
 </p>
 ------------------------------
+[[Image(...)]]
+============================== source: Image, inline
+[[Image(source:« test ».png, inline)]]
+------------------------------
+<p>
+<a style="padding:0; border:none" href="/browser/%C2%AB%20test%C2%A0%C2%BB.png"><img src="/browser/%C2%AB%20test%C2%A0%C2%BB.png?format=raw" alt="source:« test ».png" title="source:« test ».png" /></a>
+</p>
+------------------------------
 <a style="padding:0; border:none" href="/browser/%C2%AB%20test%C2%A0%C2%BB.png"><img src="/browser/%C2%AB%20test%C2%A0%C2%BB.png?format=raw" alt="source:« test ».png" title="source:« test ».png" /></a>
 ============================== intertrac:source: Image, no other arguments
 [[Image(trac:source:/trunk/doc/images/bkgnd_pattern_« test ».png)]]
@@ -29,76 +78,142 @@ IMAGE_MACRO_TEST_CASES = u"""
 <p>
 <a style="padding:0; border:none" href="http://trac.edgewall.org/intertrac/source%3A/trunk/doc/images/bkgnd_pattern_%C2%AB%20test%C2%A0%C2%BB.png"><img src="http://trac.edgewall.org/intertrac/source%3A/trunk/doc/images/bkgnd_pattern_%C2%AB%20test%C2%A0%C2%BB.png%3Fformat%3Draw" alt="source:/trunk/doc/images/bkgnd_pattern_« test ».png in Trac's Trac" title="source:/trunk/doc/images/bkgnd_pattern_« test ».png in Trac's Trac" /></a>
 </p>
-------------------------------
-<a style="padding:0; border:none" href="http://trac.edgewall.org/intertrac/source%3A/trunk/doc/images/bkgnd_pattern_%C2%AB%20test%C2%A0%C2%BB.png"><img src="http://trac.edgewall.org/intertrac/source%3A/trunk/doc/images/bkgnd_pattern_%C2%AB%20test%C2%A0%C2%BB.png%3Fformat%3Draw" alt="source:/trunk/doc/images/bkgnd_pattern_« test ».png in Trac's Trac" title="source:/trunk/doc/images/bkgnd_pattern_« test ».png in Trac's Trac" /></a>
 ============================== source: Image, nolink
 [[Image(source:« test », nolink)]]
 ------------------------------
 <p>
 <img src="/browser/%C2%AB%20test%C2%A0%C2%BB?format=raw" alt="source:« test »" title="source:« test »" />
 </p>
-------------------------------
-<img src="/browser/%C2%AB%20test%C2%A0%C2%BB?format=raw" alt="source:« test »" title="source:« test »" />
 ============================== source: Image, normal args
 [[Image(source:« test », align=left, title=Test)]]
 ------------------------------
 <p>
 <a style="padding:0; border:none" href="/browser/%C2%AB%20test%C2%A0%C2%BB"><img src="/browser/%C2%AB%20test%C2%A0%C2%BB?format=raw" alt="source:« test »" style="float:left" title="Test" /></a>
 </p>
-------------------------------
-<a style="padding:0; border:none" href="/browser/%C2%AB%20test%C2%A0%C2%BB"><img src="/browser/%C2%AB%20test%C2%A0%C2%BB?format=raw" alt="source:« test »" style="float:left" title="Test" /></a>
 ============================== source: Image, size arg
 [[Image(source:« test », 30%)]]
 ------------------------------
 <p>
 <a style="padding:0; border:none" href="/browser/%C2%AB%20test%C2%A0%C2%BB"><img width="30%" alt="source:« test »" title="source:« test »" src="/browser/%C2%AB%20test%C2%A0%C2%BB?format=raw" /></a>
 </p>
-------------------------------
 ============================== source: Image, keyword alignment
 [[Image(source:« test », right)]]
 ------------------------------
 <p>
 <a style="padding:0; border:none" href="/browser/%C2%AB%20test%C2%A0%C2%BB"><img src="/browser/%C2%AB%20test%C2%A0%C2%BB?format=raw" alt="source:« test »" style="float:right" title="source:« test »" /></a>
 </p>
-------------------------------
 ============================== http: Image, nolink
 [[Image(http://www.edgewall.com/gfx/shredder_« test ».png, nolink)]]
 ------------------------------
 <p>
 <img src="http://www.edgewall.com/gfx/shredder_« test ».png" alt="http://www.edgewall.com/gfx/shredder_« test ».png" title="http://www.edgewall.com/gfx/shredder_« test ».png" />
 </p>
-------------------------------
 ============================== http: Image, absolute, many ':'
 [[Image(http://chart.apis.google.com:80/chart?cht=p3&chd=s:hW&chs=250x100&chl=Héllo|Wôrld, title=Google & Charting, link=)]]
 ------------------------------
 <p>
 <img src="http://chart.apis.google.com:80/chart?cht=p3&amp;chd=s:hW&amp;chs=250x100&amp;chl=Héllo|Wôrld" alt="http://chart.apis.google.com:80/chart" title="Google &amp; Charting" />
 </p>
-------------------------------
 ============================== // Image, server-relative
 [[Image(//browser/« test »?format=raw, link=)]]
 ------------------------------
 <p>
 <img src="/browser/« test »?format=raw" alt="/browser/« test »" title="/browser/« test »" />
 </p>
-------------------------------
 ============================== / Image, project-relative, link to WikiStart
 [[Image(/browser/« test »?format=raw, link=wiki:WikiStart)]]
 ------------------------------
 <p>
 <a style="padding:0; border:none" href="/wiki/WikiStart"><img src="/browser/%C2%AB%20test%C2%A0%C2%BB?format=raw" alt="/browser/« test »" title="/browser/« test »" /></a>
 </p>
-------------------------------
-<a style="padding:0; border:none" href="/wiki/WikiStart"><img src="/browser/%C2%AB%20test%C2%A0%C2%BB?format=raw" alt="/browser/« test »" title="/browser/« test »" /></a>
 ============================== Strip unicode white-spaces and ZWSPs (#10668)
-[[Image(  ​source:« test ».png　 ​, nolink)]]
+[[Image(  ​source:« test ».png　 ​, nolink, 100%　 ​)]]
 ------------------------------
 <p>
-<img src="/browser/%C2%AB%20test%C2%A0%C2%BB.png?format=raw" alt="source:« test ».png" title="source:« test ».png" />
+<img width="100%" alt="source:« test ».png" title="source:« test ».png" src="/browser/%C2%AB%20test%C2%A0%C2%BB.png?format=raw" />
 </p>
+============================== Attachments on page with ':' characters (#10562)
+[[Image("page:fr":img.png​,nolink)]]
 ------------------------------
-<img src="/browser/%C2%AB%20test%C2%A0%C2%BB.png?format=raw" alt="source:« test ».png" title="source:« test ».png" />
+<p>
+<img src="/raw-attachment/wiki/page%3Afr/img.png" alt="image in page:fr" title="image in page:fr" />
+</p>
+============================== htdocs: Image, nolink
+[[Image(htdocs:trac_logo.png, nolink)]]
 ------------------------------
+<p>
+<img src="/chrome/site/trac_logo.png" alt="trac_logo.png" title="trac_logo.png" />
+</p>
+============================== shared: Image, nolink
+[[Image(shared:trac_logo.png, nolink)]]
+------------------------------
+<p>
+<img src="/chrome/shared/trac_logo.png" alt="trac_logo.png" title="trac_logo.png" />
+</p>
+==============================
+[[Image("data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACgAAAAoAQMAAAC2MCouAAAAA1BMVEXLQ0MOAUiXAAAAC0lEQVQIHWMYYQAAAPAAASEIRrcAAAAASUVORK5CYII=")]]
+------------------------------
+<p>
+<a style="padding:0; border:none" href="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACgAAAAoAQMAAAC2MCouAAAAA1BMVEXLQ0MOAUiXAAAAC0lEQVQIHWMYYQAAAPAAASEIRrcAAAAASUVORK5CYII="><img src="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACgAAAAoAQMAAAC2MCouAAAAA1BMVEXLQ0MOAUiXAAAAC0lEQVQIHWMYYQAAAPAAASEIRrcAAAAASUVORK5CYII=" alt="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACgAAAAoAQMAAAC2MCouAAAAA1BMVEXLQ0MOAUiXAAAAC0lEQVQIHWMYYQAAAPAAASEIRrcAAAAASUVORK5CYII=" title="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACgAAAAoAQMAAAC2MCouAAAAA1BMVEXLQ0MOAUiXAAAAC0lEQVQIHWMYYQAAAPAAASEIRrcAAAAASUVORK5CYII=" /></a>
+</p>
+==============================
+[[Image("data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACgAAAAoAQMAAAC2MCouAAAAA1BMVEXLQ0MOAUiXAAAAC0lEQVQIHWMYYQAAAPAAASEIRrcAAAAASUVORK5CYII=", nolink)]]
+------------------------------
+<p>
+<img src="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACgAAAAoAQMAAAC2MCouAAAAA1BMVEXLQ0MOAUiXAAAAC0lEQVQIHWMYYQAAAPAAASEIRrcAAAAASUVORK5CYII=" alt="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACgAAAAoAQMAAAC2MCouAAAAA1BMVEXLQ0MOAUiXAAAAC0lEQVQIHWMYYQAAAPAAASEIRrcAAAAASUVORK5CYII=" title="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACgAAAAoAQMAAAC2MCouAAAAA1BMVEXLQ0MOAUiXAAAAC0lEQVQIHWMYYQAAAPAAASEIRrcAAAAASUVORK5CYII=" />
+</p>
+==============================
+[[Image(notfound.png, nolink)]]
+------------------------------
+<p>
+<img src="http://assets.example.org/common/attachment.png" alt="No image &#34;notfound.png&#34; attached to WikiStart" title="No image &#34;notfound.png&#34; attached to WikiStart" />
+</p>
+==============================
+[[Image(img.png, margin-bottom=-1)]]
+------------------------------
+<p>
+<a style="padding:0; border:none" href="/attachment/wiki/WikiStart/img.png"><img src="http://assets.example.org/common/attachment.png" alt="No image &#34;img.png&#34; attached to WikiStart" style="margin-bottom: 1px" title="No image &#34;img.png&#34; attached to WikiStart" /></a>
+</p>
+==============================
+[[Image(img.png, margin-bottom=--)]]
+------------------------------
+<p>
+<div class="system-message"><strong>Invalid macro argument <tt>margin-bottom=--</tt></strong></div>
+</p>
+==============================
+[[Image(img.png, margin-top=--)]]
+------------------------------
+<p>
+<div class="system-message"><strong>Invalid macro argument <tt>margin-top=--</tt></strong></div>
+</p>
+==============================
+[[Image(img.png, margin=--)]]
+------------------------------
+<p>
+<div class="system-message"><strong>Invalid macro argument <tt>margin=--</tt></strong></div>
+</p>
+==============================
+[[Image(img.png, margin-left=--)]]
+------------------------------
+<p>
+<div class="system-message"><strong>Invalid macro argument <tt>margin-left=--</tt></strong></div>
+</p>
+==============================
+[[Image(img.png, margin-right=--)]]
+------------------------------
+<p>
+<div class="system-message"><strong>Invalid macro argument <tt>margin-right=--</tt></strong></div>
+</p>
+==============================
+[[Image(img.png, border=--)]]
+------------------------------
+<p>
+<div class="system-message"><strong>Invalid macro argument <tt>border=--</tt></strong></div>
+</p>
+==============================  # Regression test for #12333
+= [[Image]]
+------------------------------
+<h1 id="Image">[[Image]]</h1>
 """
 
 # Note: in the <img> src attribute above, the Unicode characters
@@ -110,14 +225,6 @@ IMAGE_MACRO_TEST_CASES = u"""
 
 # == [[TitleIndex]]
 
-def add_pages(tc, names):
-    now = datetime.now(utc)
-    for name in names:
-        w = WikiPage(tc.env)
-        w.name = name
-        w.text = '--'
-        w.save('joe', 'the page ' + name, '::1', now)
-
 def titleindex_teardown(tc):
     tc.env.reset_db()
 
@@ -140,6 +247,19 @@ TITLEINDEX1_MACRO_TEST_CASES = u"""
 </p>
 ------------------------------
 [[TitleIndex(...)]]
+==============================
+[[TitleIndex(min=--)]]
+------------------------------
+<p>
+<div class="system-message"><strong>Invalid macro argument <tt>min=--</tt></strong></div>
+</p>
+==============================
+[[TitleIndex(depth=--)]]
+------------------------------
+<p>
+<div class="system-message"><strong>Invalid macro argument <tt>depth=--</tt></strong></div>
+</p>
+------------------------------
 """
 
 TITLEINDEX2_MACRO_TEST_CASES = u"""
@@ -226,6 +346,13 @@ TITLEINDEX3_MACRO_TEST_CASES = u"""
 </p><div class="titleindex"><ul><li><a href="/wiki/WikiStart/First">First</a></li><li><a href="/wiki/WikiStart/Second">Second</a></li><li><a href="/wiki/WikiStart/Third">Third</a></li></ul></div><p>
 </p>
 ------------------------------
+============================== TitleIndex, top-level pages only
+[[TitleIndex(depth=0)]]
+------------------------------
+<p>
+</p><div class="titleindex"><ul><li><a href="/wiki/WikiStart">WikiStart</a></li></ul></div><p>
+</p>
+------------------------------
 """
 
 def titleindex3_setup(tc):
@@ -361,13 +488,19 @@ RECENTCHANGES_MACRO_TEST_CASES = u""""
 </p><div><ul><li><a href="/wiki/WikiEnd">WikiEnd</a>
 </li></ul></div><p>
 </p>
+==============================
+[[RecentChanges(Trac, --)]]
+------------------------------
+<p>
+<div class="system-message"><strong>Invalid macro argument <tt>--</tt></strong></div>
+</p>
 ------------------------------
 """
 
 def recentchanges_setup(tc):
     def add_pages(tc, names):
         for name in names:
-            now = datetime.now(utc)
+            now = datetime_now(utc)
             w = WikiPage(tc.env)
             w.name = name
             w.text = '--'
@@ -383,16 +516,141 @@ def recentchanges_teardown(tc):
     tc.env.reset_db()
 
 
+PAGEOUTLINE_MACRO_TEST_CASES = u""""\
+==============================
+[[PageOutline(a)]]
+------------------------------
+<p>
+<div class="system-message"><strong>Invalid macro argument <tt>a</tt></strong></div>
+</p>
+==============================
+[[PageOutline(a-b)]]
+------------------------------
+<p>
+<div class="system-message"><strong>Invalid macro argument <tt>a</tt></strong></div>
+</p>
+==============================
+[[PageOutline(0)]]
+= Heading Level 1
+== Heading Level 2
+------------------------------
+<p>
+</p><div class="wiki-toc">
+<ol>
+  <li>
+    <a href="#HeadingLevel1">Heading Level 1</a>
+  </li>
+</ol>
+</div><p>
+</p>
+<h1 id="HeadingLevel1">Heading Level 1</h1>
+<h2 id="HeadingLevel2">Heading Level 2</h2>
+==============================
+[[PageOutline(7)]]
+===== Heading Level 5
+====== Heading Level 6
+------------------------------
+<p>
+</p><div class="wiki-toc">
+                    <ol>
+                      <li>
+                        <a href="#HeadingLevel6">Heading Level 6</a>
+                      </li>
+                    </ol>
+</div><p>
+</p>
+<h5 id="HeadingLevel5">Heading Level 5</h5>
+<h6 id="HeadingLevel6">Heading Level 6</h6>
+==============================
+[[PageOutline(0-7)]]
+= Heading Level 1
+== Heading Level 2
+=== Heading Level 3
+==== Heading Level 4
+===== Heading Level 5
+====== Heading Level 6
+------------------------------
+<p>
+</p><div class="wiki-toc">
+<ol>
+  <li>
+    <a href="#HeadingLevel1">Heading Level 1</a>
+    <ol>
+      <li>
+        <a href="#HeadingLevel2">Heading Level 2</a>
+        <ol>
+          <li>
+            <a href="#HeadingLevel3">Heading Level 3</a>
+            <ol>
+              <li>
+                <a href="#HeadingLevel4">Heading Level 4</a>
+                <ol>
+                  <li>
+                    <a href="#HeadingLevel5">Heading Level 5</a>
+                    <ol>
+                      <li>
+                        <a href="#HeadingLevel6">Heading Level 6</a>
+                      </li>
+                    </ol>
+                  </li>
+                </ol>
+              </li>
+            </ol>
+          </li>
+        </ol>
+      </li>
+    </ol>
+  </li>
+</ol>
+</div><p>
+</p>
+<h1 id="HeadingLevel1">Heading Level 1</h1>
+<h2 id="HeadingLevel2">Heading Level 2</h2>
+<h3 id="HeadingLevel3">Heading Level 3</h3>
+<h4 id="HeadingLevel4">Heading Level 4</h4>
+<h5 id="HeadingLevel5">Heading Level 5</h5>
+<h6 id="HeadingLevel6">Heading Level 6</h6>
+"""
+
+
 TRACINI_MACRO_TEST_CASES = u"""\
 ============================== TracIni, option with empty doc (#10940)
 [[TracIni(section-42)]]
 ------------------------------
 <p>
-</p><div class="tracini">\
-<h3 id="section-42-section"><code>[section-42]</code></h3>\
+</p><div class="tracini"><h3 id="section-42-section"><code>[section-42]</code></h3>\
 <table class="wiki"><tbody>\
-<tr><td><tt>option1</tt></td><td></td><td class="default"><code>value</code></td></tr>\
-<tr><td><tt>option2</tt></td><td>blah</td><td class="default"><code>value</code></td></tr>\
+<tr class="even"><td><tt>option1</tt></td><td></td><td class="default"><code>value</code></td></tr>\
+<tr class="odd"><td><tt>option2</tt></td><td><p>
+blah
+</p>
+</td><td class="default"><code>value</code></td></tr></tbody></table></div><p>
+</p>
+------------------------------
+============================== TracIni, list option with sep=| (#11074)
+[[TracIni(section-list)]]
+------------------------------
+<p>
+</p><div class="tracini">\
+<h3 id="section-list-section"><code>[section-list]</code></h3>\
+<table class="wiki"><tbody>\
+<tr class="even"><td><tt>option1</tt></td><td></td><td class="default"><code>4.2|42|42||0|enabled</code></td></tr>\
+</tbody></table>\
+</div><p>
+</p>
+------------------------------
+============================== TracIni, option with "false" value as default
+[[TracIni(section-def)]]
+------------------------------
+<p>
+</p><div class="tracini">\
+<h3 id="section-def-section"><code>[section-def]</code></h3>\
+<table class="wiki"><tbody>\
+<tr class="even"><td><tt>option1</tt></td><td></td><td class="nodefault">(no default)</td></tr>\
+<tr class="odd"><td><tt>option2</tt></td><td></td><td class="nodefault">(no default)</td></tr>\
+<tr class="even"><td><tt>option3</tt></td><td></td><td class="default"><code>0</code></td></tr>\
+<tr class="odd"><td><tt>option4</tt></td><td></td><td class="default"><code>disabled</code></td></tr>\
+<tr class="even"><td><tt>option5</tt></td><td></td><td class="default"><code></code></td></tr>\
 </tbody></table>\
 </div><p>
 </p>
@@ -404,6 +662,13 @@ def tracini_setup(tc):
     class Foo(object):
         option_a1 = (Option)('section-42', 'option1', 'value', doc='')
         option_a2 = (Option)('section-42', 'option2', 'value', doc='blah')
+        option_l1 = (ListOption)('section-list', 'option1',
+                                 [4.2, '42', 42, None, 0, True], sep='|')
+        option_d1 = (Option)('section-def', 'option1', None)
+        option_d2 = (Option)('section-def', 'option2', '')
+        option_d3 = (IntOption)('section-def', 'option3', 0)
+        option_d4 = (BoolOption)('section-def', 'option4', False)
+        option_d5 = (ListOption)('section-def', 'option5', [])
 
 def tracini_teardown(tc):
     Option.registry = tc._orig_registry
@@ -411,7 +676,9 @@ def tracini_teardown(tc):
 
 def suite():
     suite = unittest.TestSuite()
-    suite.addTest(formatter.suite(IMAGE_MACRO_TEST_CASES, file=__file__))
+    suite.addTest(formatter.suite(IMAGE_MACRO_TEST_CASES, file=__file__,
+                                  setup=image_setup,
+                                  teardown=image_teardown))
     suite.addTest(formatter.suite(TITLEINDEX1_MACRO_TEST_CASES, file=__file__))
     suite.addTest(formatter.suite(TITLEINDEX2_MACRO_TEST_CASES, file=__file__,
                                   setup=titleindex2_setup,
@@ -428,6 +695,7 @@ def suite():
     suite.addTest(formatter.suite(RECENTCHANGES_MACRO_TEST_CASES, file=__file__,
                                   setup=recentchanges_setup,
                                   teardown=recentchanges_teardown))
+    suite.addTest(formatter.suite(PAGEOUTLINE_MACRO_TEST_CASES, file=__file__))
     suite.addTest(formatter.suite(TRACINI_MACRO_TEST_CASES, file=__file__,
                                   setup=tracini_setup,
                                   teardown=tracini_teardown))
